@@ -1,6 +1,6 @@
 package Resque::Failures;
 # ABSTRACT: Class for managing Resque failures
-$Resque::Failures::VERSION = '0.22';
+$Resque::Failures::VERSION = '0.23';
 use Moose;
 with 'Resque::Encoder';
 use Class::Load qw(load_class);
@@ -9,6 +9,7 @@ use Carp;
 has resque => (
     is       => 'ro',
     required => 1,
+    weak_ref => 1,
     handles  => [qw/ redis key /]
 );
 
@@ -68,10 +69,7 @@ sub requeue {
 
 sub _requeue {
     my ( $self, $item, $queue ) = @_;
-    $self->resque->push( $queue || $item->{queue} => {
-        class => $item->{payload}{class},
-        args  => $item->{payload}{args},
-    });
+    $self->resque->push( $queue || $item->{queue} => $item->{payload} );
 }
 
 sub remove {
@@ -96,12 +94,14 @@ sub mass_remove {
         my $item = $enc->decode($encoded_item);
 
         my $match = (!$opt{queue} && !$opt{error} && !$opt{class} && !$opt{args})
-                 || ($opt{queue} && $item->{queue} =~ $opt{queue})
-                 || ($opt{error} && $item->{error} =~ $opt{error})
-                 || ($opt{class} && $item->{payload}{class} =~ $opt{class})
-                 || ($opt{args}  && $enc->encode($item->{payload}{args}) =~ $opt{args});
+                 || (
+                     (!$opt{queue} || $item->{queue} =~ $opt{queue})
+                     && (!$opt{error} || $item->{error} =~ $opt{error})
+                     && (!$opt{class} || $item->{payload}{class} =~ $opt{class})
+                     && (!$opt{args}  || $enc->encode($item->{payload}{args}) =~ $opt{args})
+                 );
 
-        if ( $match ) { $rem++; $self->_requeue($item, $opt{queue}) if $opt{requeue} }
+        if ( $match ) { $rem++; $self->_requeue($item, $opt{requeue_to}) if $opt{requeue} || $opt{requeue_to} }
         else          { $self->redis->rpush( $key => $encoded_item ) }
 
         last if ++$count >= $opt{limit};
@@ -124,7 +124,7 @@ Resque::Failures - Class for managing Resque failures
 
 =head1 VERSION
 
-version 0.22
+version 0.23
 
 =head1 ATTRIBUTES
 
@@ -193,8 +193,8 @@ doesn't match will be placed back at the end of the failed jobs.
 
 The behavior can be modified with the following options:
 
-    requeue: requeue matching jobs after being removed
-    queue:   force requeued jobs to be placed on this queue
+    requeue: requeue matching jobs after being removed.
+    requeue_to: Requeued jobs will be placed on this queue instead of the original one. This option implies requeue.
 
 Example
 
